package com.tenlionsoft.aimz_k.viewmodel

import android.content.Context
import android.content.Intent
import android.util.Log
import android.view.View
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.viewModelScope
import com.atwa.filepicker.result.FileMeta
import com.atwa.filepicker.result.ImageMeta
import com.atwa.filepicker.result.VideoMeta
import com.google.gson.Gson
import com.tenlionsoft.aimz_k.ConvertBeanUtils
import com.tenlionsoft.aimz_k.adapter.ChatMsgAdapter
import com.tenlionsoft.aimz_k.model.BodyContent
import com.tenlionsoft.aimz_k.model.CoverSealedBean
import com.tenlionsoft.aimz_k.model.DbManager
import com.tenlionsoft.aimz_k.model.MsgBean
import com.tenlionsoft.aimz_k.model.MsgConvertBean
import com.tenlionsoft.aimz_k.model.PickerType
import com.tenlionsoft.aimz_k.model.Receiver
import com.tenlionsoft.aimz_k.model.Sender
import com.tenlionsoft.baselib.base.BaseViewModel
import com.tenlionsoft.baselib.contacts.ProjectConfig
import com.tenlionsoft.baselib.utils.SpUtils
import com.tenlionsoft.baselib.utils.TimeUtils
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

class ChatPageViewModel(
    private val fromId: String,
    private val toId: String,
    private val context: Context
) : BaseViewModel() {
    val txtMsg = MutableLiveData<String>("")
    val showSendBtn = MutableLiveData(false)//显示/隐藏发送按钮
    val showEmojiLayout = MutableLiveData(false)//显示/隐藏emoji
    val showReplyLayout = MutableLiveData(false)//显示/隐藏快速回复
    val showChooseLayout = MutableLiveData(false)//显示/隐藏选择
    val chooseType = MutableLiveData<PickerType>()//选择文件类型
    val scrollListToBottom = MutableLiveData<Boolean>(false)
    private val _msgList = MutableLiveData<List<MsgBean>>()
    var adapter: ChatMsgAdapter = ChatMsgAdapter(_msgList.value ?: emptyList(), this)
    private val mGson: Gson = Gson()

    init {
        Log.e("ChatPageViewModel", "Init: ${showSendBtn.value}")
        viewModelScope.launch {
            val list = getList()
            Log.e("ChatPageViewModel", "Init:${list} ")
            _msgList.value = list
            adapter.setData(list)
            scrollListToBottom.value = true
        }
    }

    //获取列表
    private suspend fun getList(): List<MsgBean> {
        return withContext(Dispatchers.IO) {
            val msgDao = DbManager.db.msgDao()
            return@withContext msgDao.getMsgByFromOrToPage(fromId, toId)
        }
    }

    fun onTxtChange(s: CharSequence, start: Int, before: Int, count: Int) {
        this.txtMsg.value = s.toString()
        showSendBtn.value = s.isNotEmpty()
    }

    fun onShowReplyLayout() {
        showReplyLayout.value = !showReplyLayout.value!!
        showEmojiLayout.value = false
        showChooseLayout.value = false
        showSoftKeyboard.value = !showReplyLayout.value!!
    }

    /**
     * 切换emoji layout
     */
    fun onShowEmojiLayout() {
        showEmojiLayout.value = !showEmojiLayout.value!!
        showChooseLayout.value = false
        showReplyLayout.value = false
        showSoftKeyboard.value = !showEmojiLayout.value!!
    }


    /**
     * 切换选择layout
     */
    fun onShowChooseLayout() {
        showChooseLayout.value = !showChooseLayout.value!!
        showEmojiLayout.value = false
        showReplyLayout.value = false
        showSoftKeyboard.value = !showChooseLayout.value!!
    }

    /**
     * 去选择图片
     */
    fun showChoosePic() {
        chooseType.value = PickerType.TYPE_PIC
    }

    /**
     * 去选择视频
     */
    fun showChooseVideo() {
        chooseType.value = PickerType.TYPE_VIDEO
    }

    /**
     * 去选择文件
     */
    fun showChooseFile() {
        chooseType.value = PickerType.TYPE_FILE
    }

    /**
     * 上传图片
     */
    fun uploadImages(list: List<ImageMeta?>) {

    }

    /**
     * 上传视频
     */
    fun uploadVideo(meta: VideoMeta) {

    }

    /**
     * 上传文件
     */
    fun uploadFiles(meta: FileMeta) {

    }

    //刷新
    private fun refresh() {
        viewModelScope.launch {
            val list = getList()
            _msgList.value = list
            adapter.setData(_msgList.value!!)
        }
    }

    //刷新页面
    private fun addBeanToView(msgBean: MsgBean) {
        _msgList.value = _msgList.value?.plus(msgBean)
        adapter.setData(_msgList.value!!)
    }

    /**
     * 发送文本信息
     */
    fun sendTxtMsg(v: View) {
        if (txtMsg.value!!.isNotEmpty()) {
            val intent = Intent(ProjectConfig.A_S_MSG_SEND)
            val b = buildSendBean(ProjectConfig.MSG_TEXT)
            viewModelScope.launch {
                val msgBean = insertMsgBeanToDb(b)
                _msgList.value = _msgList.value?.plus(msgBean)
                adapter.setData(_msgList.value!!)
            }
            intent.putExtra("msgBean", b)
            v.context.sendBroadcast(intent)
            txtMsg.value = ""
            scrollListToBottom.value = true
        }
    }

    private suspend fun insertMsgBeanToDb(b: MsgConvertBean): MsgBean {
        return withContext(Dispatchers.IO) {
            val dao = DbManager.db.msgDao()
            val cDao = DbManager.db.categoryDao()
            val msgBean = ConvertBeanUtils.convertBeanToMsgBean(b)
            cDao.updateCategory(b.receiver.receiverId!!, b.status, b.body, b.timestamp)
            dao.insertMsg(msgBean)
            return@withContext msgBean
        }
    }

    //收到消息
    fun receiveMsg(msg: String?) {
        if (!msg.isNullOrEmpty()) {
            val bean = ConvertBeanUtils.covertBean(msg)
            when (bean) {
                //消息状态
                is CoverSealedBean.StateBean -> {
                    viewModelScope.launch {
                        withContext(Dispatchers.IO) {
                            val dao = DbManager.db.msgDao()
                            val cDao = DbManager.db.categoryDao()
                            val statusType =
                                mGson.fromJson(bean.data.body, BodyContent::class.java).statusType!!
                            dao.updateStatus(bean.data.messageId, statusType)
                            cDao.updateStatus(bean.data.sender.senderId!!, statusType)
                        }
                        refresh()
                    }
                }
                //新消息
                is CoverSealedBean.Msg -> {
                    //插入信息表
                    //并更新category表
                    viewModelScope.launch {
                        withContext(Dispatchers.IO) {
                            val msgDao = DbManager.db.msgDao()
                            val cDao = DbManager.db.categoryDao()
                            msgDao.insertMsg(bean.msgBean)
                            cDao.updateOrInsert(bean.msgBean.senderId!!, bean.msgBean)
                        }
                        addBeanToView(bean.msgBean)
                        sendStatusMsg(bean.msgBean.messageId!!)
                    }
                    //刷新列表
                    scrollListToBottom.value = true
                }
            }
        }
    }

    private fun sendStatusMsg(msgId: String) {
        val b = buildStatusBean(msgId)
        val intent = Intent(ProjectConfig.A_S_MSG_SEND)
        intent.putExtra("msgBean", b)
        context.sendBroadcast(intent)
    }

    //构建发送实体
    private fun buildSendBean(type: String): MsgConvertBean {
        val bodyBean = BodyContent(content = txtMsg.value, null, null)
        val body = mGson.toJson(bodyBean)
        return MsgConvertBean(
            body = body,
            customMessageType = "",
            messageId = TimeUtils.getNowDateMillis().toString(),
            messageType = type,
            metadata = "",
            timestamp = TimeUtils.getNowDateMillis(),
            sender = Sender(
                senderId = SpUtils.getId(),
                senderType = ""
            ),
            receiver = Receiver(
                receiverId = fromId,
                receiverType = "SINGLE_USER"
            ),
            status = ProjectConfig.MSG_SEND_ING,
        )
    }

    //构建状态信息
    private fun buildStatusBean(messageId: String): MsgConvertBean {
        val bodyBean = BodyContent(content = null, msg = "", statusType = "SUCCESS_RECEIVED")
        val body = mGson.toJson(bodyBean)
        return MsgConvertBean(
            body = body,
            customMessageType = "",
            messageId = messageId,
            messageType = ProjectConfig.MSG_STATUS,
            metadata = "",
            timestamp = TimeUtils.getNowDateMillis(),
            sender = Sender(
                senderId = SpUtils.getId(),
                senderType = ""
            ),
            receiver = Receiver(
                receiverId = fromId,
                receiverType = "SINGLE_USER"
            ),
            status = "",
        )
    }
}