package com.tenlionsoft.aimz_k.viewmodel

import android.content.Context
import android.content.Intent
import android.util.Log
import androidx.lifecycle.MutableLiveData
import androidx.lifecycle.ViewModel
import androidx.lifecycle.viewModelScope
import com.google.gson.Gson
import com.tenlionsoft.aimz_k.ConvertBeanUtils
import com.tenlionsoft.aimz_k.adapter.CategoryAdapter
import com.tenlionsoft.aimz_k.model.BodyContent
import com.tenlionsoft.aimz_k.model.CoverSealedBean
import com.tenlionsoft.aimz_k.model.DbManager
import com.tenlionsoft.aimz_k.model.MsgCategoryBean
import com.tenlionsoft.aimz_k.model.MsgConvertBean
import com.tenlionsoft.aimz_k.model.Receiver
import com.tenlionsoft.aimz_k.model.Sender
import com.tenlionsoft.baselib.contacts.ProjectConfig
import com.tenlionsoft.baselib.utils.SpUtils
import com.tenlionsoft.baselib.utils.TimeUtils
import kotlinx.coroutines.Dispatchers
import kotlinx.coroutines.launch
import kotlinx.coroutines.withContext

class MsgViewModel : ViewModel() {
    private val _pwdList = MutableLiveData<List<MsgCategoryBean>>()
    var adapter: CategoryAdapter = CategoryAdapter(_pwdList.value ?: emptyList(), this)
    var onItemClickListener: OnItemClickListener? = null
    val isLogining = MutableLiveData(false)
    private val mGson = Gson()
    private var mCurrentPage: Int = 1
    var isHasMore: MutableLiveData<Boolean> = MutableLiveData<Boolean>(false)


    /**
     * 获取列表数据
     */
    private fun getList(isFirst: Boolean) {
        viewModelScope.launch {
            if (isFirst) {
                _pwdList.value = emptyList()
                mCurrentPage = 1
            } else {
                ++mCurrentPage
            }
            val list = getCategoryListByPage(mCurrentPage)
            if (list.isNotEmpty()) {
                isHasMore.value = false
            } else {
                isHasMore.value = true
            }
            val updateData: List<MsgCategoryBean> = _pwdList.value!! + list
            _pwdList.value = updateData
            adapter.setData(_pwdList.value!!)
        }
    }

    private suspend fun getCategoryListByPage(page: Int): List<MsgCategoryBean> {
        var tempList: List<MsgCategoryBean>
        withContext(Dispatchers.IO) {
            val cDao = DbManager.db.categoryDao()
            tempList = cDao.getCategoryPage(10, page)
        }
        return tempList
    }

    /**
     * 加载更多
     */
    fun loadMore() {
        getList(false)
    }

    /**
     * 登陆websocket
     */
    fun doLoginWebsocket() {
        isLogining.value = true
    }

    //接收到信息
    fun fromMsg(msg: String, context: Context) {
        Log.e("MsgViewModel", "fromMsg: ${msg}")
        if (msg.isNotEmpty()) {
            val bean = ConvertBeanUtils.covertBean(msg)
            when (bean) {
                //状态消息
                is CoverSealedBean.StateBean -> {
                    viewModelScope.launch {
                        withContext(Dispatchers.IO) {
                            val dao = DbManager.db.msgDao()
                            val cDao = DbManager.db.categoryDao()
                            val statusType =
                                mGson.fromJson(bean.data.body, BodyContent::class.java).statusType!!
                            dao.updateStatus(bean.data.messageId, statusType)
                            cDao.updateStatus(bean.data.sender.senderId!!, statusType)
                        }
                        //刷新
                        refresh()
                    }
                }
                //消息
                is CoverSealedBean.Msg -> {
                    //插入信息表
                    //并更新category表
                    viewModelScope.launch {
                        withContext(Dispatchers.IO) {
                            val msgDao = DbManager.db.msgDao()
                            val cDao = DbManager.db.categoryDao()
                            msgDao.insertMsg(bean.msgBean)
                            cDao.updateOrInsert(bean.msgBean.senderId!!, bean.msgBean)
                        }
                        sendStatusMsg(bean.msgBean.messageId!!, bean.msgBean.senderId!!, context)
                        //刷新列表
                        refresh()
                    }
                }
            }
        }
    }

    private fun sendStatusMsg(msgId: String, fromId: String, context: Context) {
        val b = buildStatusBean(msgId, fromId)
        val intent = Intent(ProjectConfig.A_S_MSG_SEND)
        intent.putExtra("msgBean", b)
        context.sendBroadcast(intent)
    }

    //构建状态信息
    private fun buildStatusBean(messageId: String, fromId: String): MsgConvertBean {
        val bodyBean = BodyContent(content = null, msg = "", statusType = "SUCCESS_RECEIVED")
        val body = mGson.toJson(bodyBean)
        return MsgConvertBean(
            body = body,
            customMessageType = "",
            messageId = messageId,
            messageType = ProjectConfig.MSG_STATUS,
            metadata = "",
            timestamp = TimeUtils.getNowDateMillis(),
            sender = Sender(
                senderId = SpUtils.getId(),
                senderType = ""
            ),
            receiver = Receiver(
                receiverId = fromId,
                receiverType = "SINGLE_USER"
            ),
            status = "",
        )
    }

    /**
     * 刷新
     */
    fun refresh() {
        getList(true)
    }

    interface OnItemClickListener {
        fun onItemClick(msgCategoryBean: MsgCategoryBean)
    }
}