package com.tenlionsoft.baselib.net

import android.content.Context
import com.tenlionsoft.baselib.contacts.NetConfig
import okhttp3.OkHttpClient
import retrofit2.Retrofit
import retrofit2.adapter.rxjava3.RxJava3CallAdapterFactory
import retrofit2.converter.gson.GsonConverterFactory
import java.util.concurrent.TimeUnit

open class RetrofitClient {
    private var mCtx: Context? = null;
    private val TIMEOUT: Long = 5;//超时时间
    private lateinit var mRetrofit: Retrofit;

    private constructor(ctx: Context) {
        mCtx = ctx;
        initManager();
    }

    /**
     * 初始化Retrofit
     */
    private fun initManager() {
        val client = getHttpClient();
        mRetrofit = Retrofit.Builder()
            .baseUrl(NetConfig.MAIN_URL)
            .addConverterFactory(GsonConverterFactory.create())
            .addCallAdapterFactory(RxJava3CallAdapterFactory.create())
            .client(client)
            .build();
    }


    private fun getHttpClient(): OkHttpClient {
        val builder: OkHttpClient.Builder = OkHttpClient.Builder();
//        setLocalProxy();
        // 设置超时
        builder.connectTimeout(TIMEOUT, TimeUnit.MINUTES);
        builder.readTimeout(TIMEOUT, TimeUnit.MINUTES);
        builder.writeTimeout(TIMEOUT, TimeUnit.MINUTES);
        //封装公共参数
//        builder.addInterceptor(new AreaInterceptor());
        //builder.addInterceptor(new CommInterceptor());
        //多BaseUrl连接器
        builder.addInterceptor(BaseUrlInterceptor());
        return builder.build();
    }

    /**
     * 创建
     */
    fun <T> create(clz: Class<T>): T {
        return mRetrofit.create(clz);
    }

    companion object {
        @Volatile
        private var instance: RetrofitClient? = null;
        fun getInstance(ctx: Context) = instance ?: synchronized(this) {
            instance ?: RetrofitClient(ctx).also { instance = it }
        }
    }
}